import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import quad
import scipy
import pandas as pd
import os
from scipy.signal import savgol_filter

filename_ref = "2022-06-15 054440 Interferograms FTIR_A_D0346_pump_FEL_24p45um_WL_delay_10ns_60mW_chA_ZIH_Si_ref_end.txt"

filename = "2022-06-15 031346 Interferograms FTIR_A_D0346_pump_FEL_24p45um_WL_delay_10ns_27mW_chA_ZIH_TFlinescan_3.txt"
folder_name  = "Normalised to end TF3"

wavenumber_min = 800        # 14,29 micrometer  Re(permRef)= -7673,3
wavenumber_max = 1400       #  6,67 micrometer  Re(permRef)= -2025,4

delay_min      = 840       #  for delay range 
delay_max      = 890      #  for dealy range

delay_zero     = 852       #  for unpumped spectra averaging


### FFT parameters
dx = 800e-6                 # meter interferometer distance
sr = 1024                   # sampling rate
c  = 2.998e8                # m/s       
dt = 2*dx/(c*sr)
freq = np.fft.fftfreq(sr, dt)       # creating array of frequincies used in FFT
wavenumber = freq*33.35641/1e12     # conversion to wavenumbers

# simulation initial parameters
permittivity = 11.5 # optic permitiviy of sample
omega_plasmon = 1100 # inv. cm
omega_plasmon_limit = 1200 # np.inf for normal case
damping = 178 # inv. cm

# AFM
tipRadius = 25*10**(-9) #m
alpha_t = 4*np.pi*tipRadius**3
h_0 = 99*10**(-9) #m
oszFreq = 255*10**3 #Hz

permRef = 11.7 + 0.001*1j # permittivity of reference sample (Si)
# permRef = -5000 + 2000*1j # permittivity of reference sample (Au)
permSi  = 11.7 + 0.001*1j
harmonicFaktor = 2

# constants for reflection calculation
sin60 = np.sin(np.radians(60))
cos60 = np.cos(np.radians(60))
phase_shift = 0.2 #radians


############## Reference calculation ########################################
data = open(filename_ref, "r") # open data file

header = True
lineCount = 0

# loop over all lines in txt file until you find header, numberOfColumns and numberOfLines
with open(filename_ref, "r") as data:
  line = data.readline()
  while line:
    data.tell() #returns the location of the next line
    line = data.readline()

    lineCount += 1 # counting lines until data starts
    if("#" not in line):
        
        if(header == True):
            startingLine = data.tell()     # save number of starting line. 
            lineCount = 0                  # reset lineCounter
            headerArray = line.split("\t") #split data at tap
            header = False
        
               
# create a data array now that size in known
dataArray = np.zeros((len(headerArray), lineCount))    


lineCount = 0 # reset line counter
# open file again
with open(filename_ref, "r") as data:
  data.seek(startingLine) # jump back to line where data starts
  line = data.readline()
  while line:
    line = data.readline()
    line = line.replace("\n", "")
    linedata = line.split("\t")
    
    # fill dataarray 
    for col in range(0,len(linedata)-1,1):
        dataArray[col, lineCount] = float(linedata[col])
    
    lineCount += 1

data.close()

row   = dataArray[0]
column = dataArray[1]
run   = dataArray[2]
depth = dataArray[3]
M   = dataArray[4]
iO0A = dataArray[5]
iO0P = dataArray[6]
iO1A = dataArray[7]
iO1P = dataArray[8]
iO2A = dataArray[9]
iO2P = dataArray[10]
iO3A = dataArray[11]
iO3P = dataArray[12]
iO4A = dataArray[13]
iO4P = dataArray[14]

iO2R = np.zeros((len(iO2P))) 


# binarization of the phase and O2R calculation
for i in range(0,len(iO2P), 1):
    if (iO2P[i] < 0):
        iO2R[i] = 1*iO2A[i]
    else: 
        iO2R[i] = -1*iO2A[i]

# save all arrays in the DataFrame for pivot
dataTable_ref = pd.DataFrame({
    "row":   dataArray[0],
    "column":dataArray[1],
    "run":   dataArray[2],
    "depth": dataArray[3],
    "M":     dataArray[4],
    "iO0A":   dataArray[5],
    "iO0P":   dataArray[6],
    "iO1A":   dataArray[7],
    "iO1P":   dataArray[8],
    "iO2A":   dataArray[9],
    "iO2P":   dataArray[10],
    "iO3A":   dataArray[11],
    "iO3P":   dataArray[12],
    "iO4A":   dataArray[13],
    "iO4P":   dataArray[14],
    "iO2R":   iO2R })


# pivot data to average reference
pT_ref = pd.pivot_table(dataTable_ref, values= 'iO2R', index="row", columns= 'depth', aggfunc=np.mean) # pT is short of Pivot Table
pT_ref = pT_ref.fillna(0)      # replacing NaN values to zeros


# FFT calculation
fft_ref =  np.fft.fft(pT_ref.to_numpy())                      

# cut data to size according to given wavelength intervall
for i in range(0,sr-1, 1):
    if(wavenumber[i] < wavenumber_min): minIdx = i # find index of min Value
    if(wavenumber[i] > wavenumber_max): # find index of max value
        maxIdx = i
        break
wavenumber = wavenumber[minIdx:maxIdx]

ref = fft_ref[0,minIdx:maxIdx]

ref0 = np.abs(ref)*np.exp(1j*np.angle(ref))


### load in datafile #########################################################
data = open(filename, "r") # open data file

header = True
lineCount = 0

# loop over all lines in txt file until you find header, numberOfColumns and numberOfLines
with open(filename, "r") as data:
  line = data.readline()
  while line:
    data.tell() #returns the location of the next line
    line = data.readline()

    lineCount += 1 # counting lines until data starts
    if("#" not in line):
        
        if(header == True):
            startingLine = data.tell()     # save number of starting line. 
            lineCount = 0                  # reset lineCounter
            headerArray = line.split("\t") #split data at tap
            header = False
        
            
   
# create a data array now that size in known
dataArray = np.zeros((len(headerArray), lineCount))    


lineCount = 0 # reset line counter
# open file again
with open(filename, "r") as data:
  data.seek(startingLine) # jump back to line where data starts
  line = data.readline()
  while line:
    line = data.readline()
    line = line.replace("\n", "")
    linedata = line.split("\t")
    # fill dataarray 
    for col in range(0,len(linedata)-1,1):
        dataArray[col, lineCount] = float(linedata[col])
    
    lineCount += 1

data.close()

row   = dataArray[0]
delay = dataArray[1]
run   = dataArray[2]
depth = dataArray[3]
M   = dataArray[4]
iO0A = dataArray[5]
iO0P = dataArray[6]
iO1A = dataArray[7]
iO1P = dataArray[8]
iO2A = dataArray[9]
iO2P = dataArray[10]
iO3A = dataArray[11]
iO3P = dataArray[12]
iO4A = dataArray[13]
iO4P = dataArray[14]

iO2R = np.zeros((len(iO2P))) 

# rounding of Time Delay for shorter naming
for i in range(0,len(delay), 1):
    delay[i] = round(delay[i])

# binarization of the phase and O2R calculation
for i in range(0,len(iO2P), 1):
    if (iO2P[i] < 0):
        iO2R[i] = 1*iO2A[i]
    else: 
        iO2R[i] = -1*iO2A[i]


# save all arrays in the DataFrame for pivot
dataTable = pd.DataFrame({
    "row":   dataArray[0],
    "delay": dataArray[1],
    "run":   dataArray[2],
    "depth": dataArray[3],
    "M":     dataArray[4],
    "iO0A":   dataArray[5],
    "iO0P":   dataArray[6],
    "iO1A":   dataArray[7],
    "iO1P":   dataArray[8],
    "iO2A":   dataArray[9],
    "iO2P":   dataArray[10],
    "iO3A":   dataArray[11],
    "iO3P":   dataArray[12],
    "iO4A":   dataArray[13],
    "iO4P":   dataArray[14],
    "iO2R":   iO2R })


# pivot data to average reference
pT = pd.pivot_table(dataTable, values= 'iO2R', index="depth", columns= 'delay', aggfunc=np.mean) # pT is short of Pivot Table
pT = pT.fillna(0)      # replacing NaN values to zeros
length = len(pT.columns.to_numpy()) # replacing NaN values to zeros


# FFT calculation
length = len(pT.columns.to_numpy())
fft = np.zeros((length, sr), dtype= complex)
for  i in range(0,length, 1):
    fft[i] = np.fft.fft(pT[pT.columns[i]])
fft_T = fft.transpose()                      #transpose of the array

O2R = pd.DataFrame(fft_T[minIdx:maxIdx,:],index=wavenumber, columns=pT.columns.to_numpy())
O2A = pd.DataFrame(0,index=wavenumber, columns=pT.columns.to_numpy())
O2P = pd.DataFrame(0,index=wavenumber, columns=pT.columns.to_numpy())

############# removing the phase shift #############################################################

# cut data to size according to given wavelength intervall
for i in range(0,len(wavenumber), 1):
    if(wavenumber[i] < 1250): minIdx = i # find index of min Value
    if(wavenumber[i] > 1400): # find index of max value
        maxIdx = i
        break
    
for i in range(0,length, 1):
    O2R[O2R.columns[i]] = O2R[O2R.columns[i]]/ref
    dP = np.average(np.angle(O2R[O2R.columns[i]])[minIdx:maxIdx])
    O2R[O2R.columns[i]] = np.abs(O2R[O2R.columns[i]])*np.exp(1j*(np.angle(O2R[O2R.columns[i]])-dP))
     

############# calculation of real pumped spectum (6'pumped - 5unpumped)########'######################

zeroO2R = np.zeros((len(O2R[O2R.columns[1]]))) 

# averaging of the first spectra

for i in range(1,length, 1):
    zeroO2R = zeroO2R + O2R[O2R.columns[i]] 
    if(O2R.columns[i] == delay_zero):           # find index of time zero 
        zeroO2R =  zeroO2R/i          # average unpumped spectra
        break
'''
###
# averaging of the last spectra
switch = False
for i in range(1,length, 1):
    if(O2R.columns[i] > delay_zero):           # find index of time zero 
        switch = True        # average unpumped spectra
    if switch:
        zeroO2R = zeroO2R + O2R[O2R.columns[i]] 
zeroO2R =  zeroO2R/(5)   
'''


###
for i in range(1,length, 1):
    O2R[O2R.columns[i]] = 6*O2R[O2R.columns[i]]-5*zeroO2R
   

####################################################################################################


#calculation of 02A and O2P
for i in range(0,length, 1):
    O2A[O2R.columns[i]] = np.abs(O2R[O2R.columns[i]])
    O2P[O2R.columns[i]] = np.angle(O2R[O2R.columns[i]])

    
### fitting of the data #######################################################

# creating txt file for fitting parameters
os.makedirs(folder_name)
with open(os.path.join(folder_name, 'fitting parameters.txt'), 'w') as f:
    f.write(' Fit parameters: \n Permittivity: ' + str(permittivity) + '\n Tip Radius:' +\
            str(tipRadius) + '\n Tip Amplitude h0:' + str(h_0) +\
            '\n Tip Freq:' + str(oszFreq) +\
            '\n Delay Wpl dWpl Y  dY PhaseSift dPhaseShift')  
        
#saving pivot data for plotting
O2A.to_csv(os.path.join(folder_name,'data O2A modified.csv'), index=True)
O2P.to_csv(os.path.join(folder_name,'data O2P modified.csv'), index=True)



### smoothing of the results and saving #######################################################
'''
for i in range(0,length, 1):
    O2A[O2R.columns[i]] = savgol_filter(O2A[O2R.columns[i]], 50, 10)
    O2P[O2R.columns[i]] = savgol_filter(O2P[O2R.columns[i]], 50, 10)
    
O2A.to_csv(os.path.join(folder_name,'data O2A modified smooth.csv'), index=True)
O2P.to_csv(os.path.join(folder_name,'data O2P modified smooth.csv'), index=True)    
'''
####################################################################################################



def permittivityFunc(k, permittivity, omega_plasmon, damping):
    return permittivity - (omega_plasmon**2 * permittivity) / (k**2 + k*damping*1j)


def scatteringCrosssection_Ref(time):
    # calulate beta
    beta = (permRef-1)/(permRef+1)
    # calulate h
    h =tipRadius + h_0 * (1 + np.cos(oszFreq * 2 * np.pi * time))
    # reflection calulation
    tauRef  = np.arcsin(sin60 / np.sqrt(permRef))
    reflectionRef = (np.sqrt(permRef)*cos60 - np.cos(tauRef)) /\
            (np.sqrt(permRef)*cos60 + np.cos(tauRef)) 

    return ((reflectionRef + 1)**2 * alpha_t)/ \
           (1 - (beta * alpha_t)/\
                (16*np.pi*h**3))

def scatteringCrosssection(perm, time):
    #perm = permittivityFunc(wavenumber)
    # calulate beta
    beta = (perm-1)/(perm+1)
    # calulate h
    h =tipRadius + h_0 * (1 + np.cos(oszFreq * 2 * np.pi * time))
    # reflection calulation
    tau  = np.arcsin(sin60 / np.sqrt(perm))
    reflection = (np.sqrt(perm)*cos60 - np.cos(tau)) /\
            (np.sqrt(perm)*cos60 + np.cos(tau))
    tau0  = np.arcsin(sin60 / np.sqrt(permSi))
    reflection0 = (np.sqrt(permSi)*cos60 - np.cos(tau0)) /\
             (np.sqrt(permSi)*cos60 + np.cos(tau0)) 
    return ((reflection0 + 1)**2 * alpha_t)/ \
           (1 - (beta * alpha_t)/\
                (16*np.pi*h**3))
        
        

def scattCrossFittingFuncPhase(wavenumber, omega_plasmon, damping, phase_shift):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    
    return phase_shift + np.angle(sampleNF/referenceNF)


def scattCrossFittingFuncAmpl(wavenumber, omega_plasmon, damping):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    
    return np.abs(sampleNF/referenceNF)


def scattCrossFittingFunc(wavenumber, omega_plasmon, damping, phase_shift):
    
    N = np.size(wavenumber)
    
    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
   
    
    result_ampl  = []
    result_phase = []
    
    for i in range(N//2):
        
        perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber[i]**2 + wavenumber[i]*damping*1j)    
        real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
        imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
        sampleNF         = real_sampleNF + imag_sampleNF*1j
        result_ampl.append(np.abs(sampleNF/referenceNF))
        result_phase.append(phase_shift + np.angle(sampleNF/referenceNF))
        
    return np.hstack([result_ampl, result_phase])


# cut data to size according to given Time Delay intervall
length = len(O2A.columns.to_numpy())
for i in range(0,length, 1):
    if(O2A.columns[i] <= delay_min): minIdx = i # find index of min Value
    if(O2A.columns[i] >= delay_max): # find index of max value
        maxIdx = i
        break

# saving of fitted curves
O2Afit = O2A
O2Pfit = O2P

for i in range(minIdx, maxIdx+1, 1):
    fig = plt.figure(figsize=(10,10))
    plt.title('2H Amplitude, Time Delay: '+str(O2A.columns[i])+' ps', fontsize=20)
    plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
    plt.plot(wavenumber, O2A[O2A.columns[i]], label=" Experiment")
    
    # vectorize the fitting function - python does not like integrals in fitting functions
    vcurve      = np.vectorize(scattCrossFittingFunc)
    vcurveAmpl  = np.vectorize(scattCrossFittingFuncAmpl)
    vcurvePhase = np.vectorize(scattCrossFittingFuncPhase)
    
    
    #combining data of phase and amplitude for the joint fitting
    jointData = np.hstack([O2A[O2A.columns[i]], O2P[O2P.columns[i]]])
    
    # curve_fit is used to fit our function to the data
    harmonicFaktor = 2
    

    popt, pcov = scipy.optimize.curve_fit(scattCrossFittingFunc, np.hstack([wavenumber, wavenumber]), jointData, p0=[omega_plasmon, damping, phase_shift],\
                                          bounds=((0, 0, -np.inf), (omega_plasmon_limit, np.inf, np.inf)))
    
    _omega_plasmon, _damping, _phase_shift      = popt
    
    d_omega_plasmon, d_damping, d_phase_shift = np.sqrt(np.diag(pcov))
    
    
    #writing fitting parameters into txt file
    with open(os.path.join(folder_name, 'fitting parameters.txt'), 'a') as f:
        f.write('\n'+str(O2P.columns[i])+' '+str(_omega_plasmon)+' '+str(d_omega_plasmon)+' '+str(_damping)+' '+str(d_damping)+\
                ' '+' '+str(_phase_shift)+' '+str(d_phase_shift))
    
     
    
    plt.plot(wavenumber, vcurveAmpl(wavenumber, _omega_plasmon, _damping), linewidth = 7, label = ' Fitting: \n Wpl: ' + str(round(_omega_plasmon)) + ' cm-1 \n Y: ' +\
             str(round(_damping)) +  ' cm-1 ', color = 'C1')
    
    plt.legend(fontsize=20)
    fig_name = 'O2A_'+str(O2A.columns[i])+'.png'
    fig_name = fig_name.replace('.','p',1)
    plt.savefig(os.path.join(folder_name, fig_name)) 
    plt.show()
    
    
    
    # plot data second harmonic ad fit Phase
    fig = plt.figure(figsize=(10,10))
    plt.title('2H Phase, Time Delay: '+str(O2A.columns[i])+' ps', fontsize=20)
    plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
    plt.plot(wavenumber, O2P[O2P.columns[i]], label=" Experiment")
        
    plt.plot(wavenumber, vcurvePhase(wavenumber, _omega_plasmon, _damping, _phase_shift), linewidth = 7, label = ' Fiting: \n Phase shift: ' + str(np.round(_phase_shift,3)), color = 'C1')
    plt.legend(fontsize=20)
    fig_name = 'O2P_'+str(O2P.columns[i])+'.png'
    fig_name = fig_name.replace('.','p',1)
    fig.savefig(os.path.join(folder_name, fig_name))
    plt.show()
    
    print(i-minIdx+1, " of ", maxIdx+1-minIdx ," done")
    
    # writing up the fit curves
    O2Afit[O2Afit.columns[i]] = vcurveAmpl(wavenumber, _omega_plasmon, _damping)
    O2Pfit[O2Pfit.columns[i]] = vcurvePhase(wavenumber, _omega_plasmon, _damping, _phase_shift) 

# saving the fit curves
O2Afit.to_csv(os.path.join(folder_name,'data O2A fit.csv'), index=True)
O2Pfit.to_csv(os.path.join(folder_name,'data O2P fit.csv'), index=True)    
    
    

##################################################

'''
# plot data of third harmonic ad fit
fig = plt.figure(figsize=(10,10))
plt.title("third Harmonic", fontsize=20)
plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
plt.plot(wavenumber, O3A, label="3H")
# vectorize the fitting function - python does not like integrals in fitting functions
vcurve = np.vectorize(scattCrossFittingFunc)
# plot vectorized function for comparison
harmonicFaktor = 3
plt.plot(wavenumber, vcurve(wavenumber, permittivity, omega_plasmon, damping), linewidth = 7, label = "simulation")

# curve_fit is used to fit our function to the data
popt, pcov = scipy.optimize.curve_fit(vcurve, wavenumber, O3A, p0=[11.4, 881, 341])

_3Hpermittivity, _3Homega_plasmon, _3Hdamping = popt
print("third Harmonic:")
print("    - permittivity: ", _3Hpermittivity)
print("    - plasmoonfeq:  ", _3Homega_plasmon)
print("    - damping:      ", _3Hdamping)

plt.plot(wavenumber, vcurve(wavenumber, _3Hpermittivity, _3Homega_plasmon, _3Hdamping), linewidth = 7, label = "fit")
plt.legend(fontsize=20)



# plot data of fourth harmonic ad fit
fig = plt.figure(figsize=(10,10))
plt.title("fourth Harmonic", fontsize=20)
plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
plt.plot(wavenumber, O4A, label="4H")

# vectorize the fitting function - python does not like integrals in fitting functions
vcurve = np.vectorize(scattCrossFittingFunc)
# plot vectorized function for comparison
plt.plot(wavenumber, vcurve(wavenumber, permittivity, omega_plasmon, damping), linewidth = 7, label = "simulation")

# curve_fit is used to fit our function to the data
harmonicFaktor = 4
popt, pcov = scipy.optimize.curve_fit(vcurve, wavenumber, O4A, p0=[11.4, 1015, 0.5])

_4Hpermittivity, _4Homega_plasmon, _4Hdamping = popt
print("fourth Harmonic:")
print("    - permittivity: ", _4Hpermittivity)
print("    - plasmoonfeq:  ", _4Homega_plasmon)
print("    - damping:      ", _4Hdamping)

plt.plot(wavenumber, vcurve(wavenumber, _4Hpermittivity, _4Homega_plasmon, _4Hdamping), linewidth = 7, label = "fit")
plt.legend(fontsize=20)



# plotting the different permittivities in comparison
perm    = permittivityFunc(wavenumber, permittivity, omega_plasmon, damping)
_2Hperm = permittivityFunc(wavenumber, _2Hpermittivity, _2Homega_plasmon, _2Hdamping)
_3Hperm = permittivityFunc(wavenumber, _3Hpermittivity, _3Homega_plasmon, _3Hdamping)
_4Hperm = permittivityFunc(wavenumber, _4Hpermittivity, _4Homega_plasmon, _4Hdamping)

fig, (real,imag)= plt.subplots(nrows=1,ncols=2)

real.set_title("real part")
real.plot(wavenumber, np.real(perm), label="simulation")
real.plot(wavenumber, np.real(_2Hperm), label="_2H")
real.plot(wavenumber, np.real(_3Hperm), label="_3H")
real.plot(wavenumber, np.real(_4Hperm), label="_4H")
real.legend()

imag.set_title("imag part")
imag.plot(wavenumber, np.imag(perm), label="simulation")
imag.plot(wavenumber, np.imag(_2Hperm), label="_2H")
imag.plot(wavenumber, np.imag(_3Hperm), label="_3H")
imag.plot(wavenumber, np.imag(_4Hperm), label="_4H")
imag.legend()
'''
